/************************************************************************

  A function to encode the MUSE Raster header record

*************************************************************************/

#include "muse.h"

#ifndef H_VECTOR		/* SR 09/25 */
/*#include "vector.h"*/
#endif

#if XVT_CC_PROTO
extern ERRSTATUS short_to_char(unsigned char *record, short *s, short big_endian, long *c);
extern ERRSTATUS ushort_to_char(unsigned char *record, unsigned short *s, short big_endian, long *c);
extern ERRSTATUS long_to_char(unsigned char *record, long *l, short big_endian, long *c);
extern ERRSTATUS double_to_char(unsigned char *record, double *d, short big_endian, long *c);
#endif

#if XVT_CC_PROTO
ERRSTATUS
encode_raster_hdr(unsigned char *record, RASTER * raster)
#else

ERRSTATUS
encode_raster_hdr(record, raster)
    unsigned char  *record;	/* the header record                    */
    RASTER         *raster;	/* the header structure                 */
#endif

{
    long            i;/* the loop counters                    */
    long            c;
    short           machine;	/* the machine type that created the file */
    short           end_check = 1;
    char           *end_char;
    short           big_endian;
    short           enum_value;
    long            ptr_value;

    /* Check for endian type */

    end_char = (char *) &end_check;
    big_endian = 1 - (short) (*end_char);

    /* Decode the values */

    c = 0;

#if XVTOS == DOSOS
    machine = 0;
#endif

#if XVTOS == MACOS
    machine = 1;
#endif

#if XVTOS == BSDOS
    machine = 2;
#endif

    short_to_char(record, &machine, big_endian, &c);

    long_to_char(record, &raster->magic, big_endian, &c);

    for (i = 0; i < 30; i++)
	record[c++] = raster->data_hdr.source[i];

    enum_value = raster->data_hdr.h_units;
    short_to_char(record, &enum_value, big_endian, &c);
    enum_value = raster->data_hdr.v_units;
    short_to_char(record, &enum_value, big_endian, &c);
    double_to_char(record, &raster->data_hdr.x_res, big_endian, &c);
    double_to_char(record, &raster->data_hdr.y_res, big_endian, &c);
    enum_value = raster->data_hdr.projection;
    short_to_char(record, &enum_value, big_endian, &c);

    double_to_char(record, &raster->data_hdr.center.lon, big_endian, &c);
    double_to_char(record, &raster->data_hdr.center.lat, big_endian, &c);

    double_to_char(record, &raster->data_hdr.extent.left, big_endian, &c);
    double_to_char(record, &raster->data_hdr.extent.right, big_endian, &c);
    double_to_char(record, &raster->data_hdr.extent.bottom, big_endian, &c);
    double_to_char(record, &raster->data_hdr.extent.top, big_endian, &c);

    enum_value = raster->data_hdr.datum.hor_datum;
    short_to_char(record, &enum_value, big_endian, &c);
    enum_value = raster->data_hdr.datum.ver_datum;
    short_to_char(record, &enum_value, big_endian, &c);

    double_to_char(record, &raster->data_hdr.accuracy.rel_hor, big_endian, &c);
    double_to_char(record, &raster->data_hdr.accuracy.abs_hor, big_endian, &c);
    double_to_char(record, &raster->data_hdr.accuracy.rel_ver, big_endian, &c);
    double_to_char(record, &raster->data_hdr.accuracy.abs_ver, big_endian, &c);

    for (i = 0; i < 3; i++)
	record[c++] = raster->data_hdr.date.year[i];
    for (i = 0; i < 3; i++)
	record[c++] = raster->data_hdr.date.month[i];
    for (i = 0; i < 3; i++)
	record[c++] = raster->data_hdr.date.day[i];
    for (i = 0; i < 3; i++)
	record[c++] = raster->data_hdr.date.hour[i];
    for (i = 0; i < 3; i++)
	record[c++] = raster->data_hdr.date.minute[i];
    for (i = 0; i < 3; i++)
	record[c++] = raster->data_hdr.date.second[i];

    double_to_char(record, &raster->data_hdr.zoom, big_endian, &c);
    double_to_char(record, &raster->data_hdr.data_min, big_endian, &c);
    double_to_char(record, &raster->data_hdr.data_max, big_endian, &c);

    long_to_char(record, &raster->data_matrix.width, big_endian, &c);
    long_to_char(record, &raster->data_matrix.height, big_endian, &c);
    long_to_char(record, &raster->data_matrix.bits_per_pixel, big_endian, &c);
    long_to_char(record, &raster->data_matrix.bytes_per_row, big_endian, &c);
    long_to_char(record, &raster->data_matrix.bytes_per_row, big_endian, &c);

    enum_value = raster->data_matrix.num_type;
    short_to_char(record, &enum_value, big_endian, &c);
    enum_value = raster->data_matrix.data_type;
    short_to_char(record, &enum_value, big_endian, &c);

    ptr_value = (raster->palette) ? (long) SZ_RASTER : (long) NULL;
    long_to_char(record, &ptr_value, big_endian, &c);
    ptr_value = (raster->lut) ? (long) SZ_LUT : (long) NULL;
    long_to_char(record, &ptr_value, big_endian, &c);
    return STAT_SUCCESS;

}

/************************************************************************

  A function to encode the MUSE LUT structure

*************************************************************************/

#if XVT_CC_PROTO
ERRSTATUS
encode_lut(unsigned char *record, LUT * lut)
#else

ERRSTATUS
encode_lut(record, lut)
    unsigned char  *record;	/* the header record                    */
    LUT            *lut;	/* the header structure                 */
#endif

{
    long            i, j;	/* the loop counters                    */
    long            c;
    short           end_check = 1;
    char           *end_char;
    short           big_endian;
    short           bool_value;

    /* Check for endian type */

    end_char = (char *) &end_check;
    big_endian = 1 - (short) (*end_char);

    /* Encode the values */

    c = 0;

    long_to_char(record, &lut->magic, big_endian, &c);

    for (i = 0; i < MAX_LUT; i++)
    {
	long_to_char(record, &lut->lut_desc[i].num_hues, big_endian, &c);
	long_to_char(record, &lut->lut_desc[i].num_shades, big_endian, &c);
	double_to_char(record, &lut->lut_desc[i].data_range[0], big_endian, &c);
	double_to_char(record, &lut->lut_desc[i].data_range[1], big_endian, &c);
	double_to_char(record, &lut->lut_desc[i].hue_range[0], big_endian, &c);
	double_to_char(record, &lut->lut_desc[i].hue_range[1], big_endian, &c);

	for (j = 0; j < 3; j++)
	{
	    double_to_char(record, &lut->lut_desc[i].shade_range[j].sat,
			   big_endian, &c);
	    double_to_char(record, &lut->lut_desc[i].shade_range[j].val,
			   big_endian, &c);
	}
    }

    for (i = 0; i < MAX_COLORS; i++)
	ushort_to_char(record, &lut->map[i], big_endian, &c);
    bool_value = lut->stretch;
    short_to_char(record, &bool_value, big_endian, &c);
    return STAT_SUCCESS;

}

/************************************************************************

  A function to encode the MUSE PALETTE structure

*************************************************************************/

#if XVT_CC_PROTO
ERRSTATUS
encode_palette(unsigned char *record, PALETTE * palette)
#else

ERRSTATUS
encode_palette(record, palette)
    unsigned char  *record;	/* the header record                    */
    PALETTE        *palette;	/* the header structure                 */
#endif

{
    long            i /* ,j */ ;/* the loop counters                    */
    long            c;
    short           end_check = 1;
    char           *end_char;
    short           big_endian;
    short           enum_value;

    /* Check for endian type */

    end_char = (char *) &end_check;
    big_endian = 1 - (short) (*end_char);

    /* Encode the values */

    c = 0;

    long_to_char(record, &palette->magic, big_endian, &c);

    enum_value = palette->type;
    short_to_char(record, &enum_value, big_endian, &c);

    /* JLL why was this here? if (palette->type == PAL_NUM_BINS) */
    {
	ushort_to_char(record, &palette->bins.r, big_endian, &c);
	ushort_to_char(record, &palette->bins.g, big_endian, &c);
	ushort_to_char(record, &palette->bins.b, big_endian, &c);
    }

    ushort_to_char(record, &palette->first_color, big_endian, &c);
    ushort_to_char(record, &palette->num_colors, big_endian, &c);

    for (i = 0; i < MAX_COLORS; i++)
    {
	ushort_to_char(record, &palette->rgb[i].r, big_endian, &c);
	ushort_to_char(record, &palette->rgb[i].g, big_endian, &c);
	ushort_to_char(record, &palette->rgb[i].b, big_endian, &c);
    }
    return STAT_SUCCESS;

}

/************************************************************************

  A function to encode the MUSE MAP_DOC structure

*************************************************************************/

#if XVT_CC_PROTO
ERRSTATUS
encode_mapdoc(unsigned char *record, MAP_DOC * mapdoc)
#else

ERRSTATUS
encode_mapdoc(record, mapdoc)
    unsigned char  *record;	/* the header record                    */
    MAP_DOC        *mapdoc;	/* the header structure                 */
#endif

{
    long            c;
     short           end_check = 1;
    char           *end_char;
    short           big_endian;
     short           bool_value;
    long            ptr_value;

    /* Check for endian type */

    end_char = (char *) &end_check;
    big_endian = 1 - (short) (*end_char);

    /* Encode the values */

    c = 0;

    long_to_char(record, &mapdoc->magic, big_endian, &c);

    ptr_value = (mapdoc->mgm) ? (long) SZ_MGM : (long) NULL;
    long_to_char(record, &ptr_value, big_endian, &c);
    ptr_value = (mapdoc->units) ? (long) SZ_MAP_DOC : (long) NULL;
    long_to_char(record, &ptr_value, big_endian, &c);
    ptr_value = (mapdoc->lut) ? (long) SZ_LUT : (long) NULL;
    long_to_char(record, &ptr_value, big_endian, &c);
    ptr_value = (mapdoc->palette) ? (long) SZ_PALETTE : (long) NULL;
    long_to_char(record, &ptr_value, big_endian, &c);
    ptr_value = (mapdoc->products) ? (long) SZ_PRODUCTS : (long) NULL;
    long_to_char(record, &ptr_value, big_endian, &c);

    bool_value = mapdoc->needs_save;
    short_to_char(record, &bool_value, big_endian, &c);
    return STAT_SUCCESS;

}

/************************************************************************

  A function to encode the MUSE MGM structure

*************************************************************************/

#if XVT_CC_PROTO
ERRSTATUS
encode_mgm(unsigned char *record, MGM * mgm)
#else

ERRSTATUS
encode_mgm(record, mgm)
    unsigned char  *record;	/* the header record                    */
    MGM            *mgm;	/* the header structure                 */
#endif

{
    long            i;/* the loop counters                    */
    long            c;
    short           end_check = 1;
    char           *end_char;
    short           big_endian;
    short           enum_value;
    short           bool_value;
    double          dummy;
    long            num_tot_params, num_params;

    /* Check for endian type */

    end_char = (char *) &end_check;
    big_endian = 1 - (short) (*end_char);

    /* Encode the values */

    c = 0;

    long_to_char(record, &mgm->magic, big_endian, &c);

    enum_value = mgm->projection;
    short_to_char(record, &enum_value, big_endian, &c);
    enum_value = mgm->hor_datum;
    short_to_char(record, &enum_value, big_endian, &c);
    enum_value = mgm->ver_datum;
    short_to_char(record, &enum_value, big_endian, &c);
    enum_value = mgm->config_pref;
    short_to_char(record, &enum_value, big_endian, &c);

    double_to_char(record, &mgm->map_center.lon, big_endian, &c);
    double_to_char(record, &mgm->map_center.lat, big_endian, &c);
    double_to_char(record, &mgm->scale_reciprocal, big_endian, &c);
    double_to_char(record, &mgm->pixel_size_on_screen, big_endian, &c);
    double_to_char(record, &mgm->pixel_size_on_ground, big_endian, &c);
    double_to_char(record, &mgm->pixel_size_lon, big_endian, &c);
    double_to_char(record, &mgm->pixel_size_lat, big_endian, &c);

    double_to_char(record, &mgm->dev_extent.left, big_endian, &c);
    double_to_char(record, &mgm->dev_extent.right, big_endian, &c);
    double_to_char(record, &mgm->dev_extent.bottom, big_endian, &c);
    double_to_char(record, &mgm->dev_extent.top, big_endian, &c);
    double_to_char(record, &mgm->dev_ymin, big_endian, &c);
    double_to_char(record, &mgm->dev_ymax, big_endian, &c);

    long_to_char(record, &mgm->xpixels, big_endian, &c);
    long_to_char(record, &mgm->ypixels, big_endian, &c);
    long_to_char(record, &mgm->bit_map_depth, big_endian, &c);

    double_to_char(record, &mgm->geo_extent.left, big_endian, &c);
    double_to_char(record, &mgm->geo_extent.right, big_endian, &c);
    double_to_char(record, &mgm->geo_extent.bottom, big_endian, &c);
    double_to_char(record, &mgm->geo_extent.top, big_endian, &c);
    double_to_char(record, &mgm->pixasp, big_endian, &c);
    double_to_char(record, &mgm->horizontal_accuracy, big_endian, &c);
    double_to_char(record, &mgm->vertical_accuracy, big_endian, &c);

    bool_value = mgm->user_aspect;
    short_to_char(record, &bool_value, big_endian, &c);

    num_tot_params = sizeof(mgm->params) / sizeof(double);
    switch (mgm->projection)
    {
      case PROJ_LINEAR:
	num_params = sizeof(LINEAR_PARAMS) / sizeof(double);
	double_to_char(record, &mgm->params.linear_params.dummy, big_endian, &c);
	break;
	
      default:
	num_params = 0;
	break;
    }
    for (i = num_params; i < num_tot_params; i++)
	double_to_char(record, &dummy, big_endian, &c);

    bool_value = mgm->valid;
    short_to_char(record, &bool_value, big_endian, &c);

    double_to_char(record, &mgm->sx, big_endian, &c);
    double_to_char(record, &mgm->sy, big_endian, &c);
    double_to_char(record, &mgm->tx, big_endian, &c);
    double_to_char(record, &mgm->ty, big_endian, &c);

    bool_value = mgm->is_locked;
    short_to_char(record, &bool_value, big_endian, &c);
    return STAT_SUCCESS;

}

/************************************************************************

  A function to encode the MUSE UNITS structure

*************************************************************************/

#if XVT_CC_PROTO
ERRSTATUS
encode_units(unsigned char *record, UNITS * units)
#else

ERRSTATUS
encode_units(record, units)
    unsigned char  *record;	/* the header record                    */
    UNITS          *units;	/* the header structure                 */
#endif

{
    long            c;
    short           end_check = 1;
    char           *end_char;
    short           big_endian;
    short           enum_value;

    /* Check for endian type */

    end_char = (char *) &end_check;
    big_endian = 1 - (short) (*end_char);

    /* Encode the values */

    c = 0;

    long_to_char(record, &units->magic, big_endian, &c);

    enum_value = units->scale;
    short_to_char(record, &enum_value, big_endian, &c);
    enum_value = units->hor_distance;
    short_to_char(record, &enum_value, big_endian, &c);
    enum_value = units->ver_distance;
    short_to_char(record, &enum_value, big_endian, &c);
    enum_value = units->area;
    short_to_char(record, &enum_value, big_endian, &c);
    enum_value = units->volume;
    short_to_char(record, &enum_value, big_endian, &c);
    enum_value = units->geographic;
    short_to_char(record, &enum_value, big_endian, &c);
    return STAT_SUCCESS;

}

/************************************************************************

  A function to encode the MUSE PRODUCTS structure

*************************************************************************/

#if XVT_CC_PROTO
ERRSTATUS
encode_products(unsigned char *record, PRODUCTS * products)
#else

ERRSTATUS
encode_products(record, products)
    unsigned char  *record;	/* the header record                    */
    PRODUCTS       *products;	/* the header structure                 */
#endif

{
    long            i, j;	/* the loop counters                    */
    long            c;
    short           end_check = 1;
    char           *end_char;
    short           big_endian;
    short           enum_value;
    long            ptr_value;

    /* Check for endian type */

    end_char = (char *) &end_check;
    big_endian = 1 - (short) (*end_char);

    /* Encode the values */

    c = 0;

    long_to_char(record, &products->magic, big_endian, &c);
    long_to_char(record, &products->bit_flags, big_endian, &c);
    short_to_char(record, &products->count, big_endian, &c);
    short_to_char(record, &products->order, big_endian, &c);

    ptr_value = (products->basemap) ? (long) SZ_BASEMAP : (long) NULL;
    long_to_char(record, &ptr_value, big_endian, &c);

    for (i = 0; i < MAX_PRODUCTS; i++)
    {
	enum_value = products->overlays[i].type;
	short_to_char(record, &enum_value, big_endian, &c);
	for (j = 0; j < 64; j++)
	    record[c++] = products->overlays[i].name[j];
	switch (enum_value)
	{
	  case GRAT:
	    ptr_value = (products->overlays[i].product_pointer) ?
		      (long) SZ_GRAT_DATA : (long) NULL;
	    break;
	  case VECT:
	    ptr_value = (products->overlays[i].product_pointer) ?
		      (long) /* SZ_VECTOR */ SZ_VEC_DATA : (long) NULL;
	    break;
	  default:
	    printf("Unknown overlay found while encoding");
	    break;
	}
	long_to_char(record, &ptr_value, big_endian, &c);
    }
    /* xvt_dm_post_note("prod c = %ld",c); */
    return STAT_SUCCESS;
}

/************************************************************************

  A function to encode the MUSE VIEW structure

*************************************************************************/

#if XVT_CC_PROTO
ERRSTATUS
encode_view(unsigned char *record, VIEW * view)
#else

ERRSTATUS
encode_view(record, view)
    unsigned char  *record;	/* the header record                    */
    VIEW           *view;	/* the header structure                 */
#endif

{
    long            c;
    short           end_check = 1;
    char           *end_char;
    short           big_endian;
    long            long_value;
    short           bool_value;
    long            ptr_value;

    /* Check for endian type */

    end_char = (char *) &end_check;
    big_endian = 1 - (short) (*end_char);

    /* Encode the values */

    c = 0;

    long_to_char(record, &view->magic, big_endian, &c);

    long_value = (long) view->bitmap_xsize;
    long_to_char(record, &long_value, big_endian, &c);
    long_value = (long) view->bitmap_ysize;
    long_to_char(record, &long_value, big_endian, &c);
    long_value = (long) view->bitmap_xmin;
    long_to_char(record, &long_value, big_endian, &c);
    long_value = (long) view->bitmap_ymin;
    long_to_char(record, &long_value, big_endian, &c);
    long_value = (long) view->bitmap_xmax;
    long_to_char(record, &long_value, big_endian, &c);
    long_value = (long) view->bitmap_ymax;
    long_to_char(record, &long_value, big_endian, &c);
    long_value = (long) view->scroll_x;
    long_to_char(record, &long_value, big_endian, &c);
    long_value = (long) view->scroll_y;
    long_to_char(record, &long_value, big_endian, &c);
    long_value = (long) view->status;
    long_to_char(record, &long_value, big_endian, &c);
    double_to_char(record, &view->zoomx, big_endian, &c);
    double_to_char(record, &view->zoomy, big_endian, &c);

    ptr_value = (long) view->winstuff;
    long_to_char(record, &ptr_value, big_endian, &c);

    bool_value = view->new_palette;
    short_to_char(record, &bool_value, big_endian, &c);
    return STAT_SUCCESS;

}

/************************************************************************

  A function to encode the MUSE DEFAULTS structure

*************************************************************************/

#if XVT_CC_PROTO
ERRSTATUS
encode_defaults(unsigned char *record, DEFAULTS * defaults)
#else

ERRSTATUS
encode_defaults(record, defaults)
    unsigned char  *record;	/* the header record                    */
    DEFAULTS       *defaults;	/* the header structure                 */
#endif

{
    long            c;
    short           end_check = 1;
    char           *end_char;
    short           big_endian;
    short           enum_value;

    /* Check for endian type */

    end_char = (char *) &end_check;
    big_endian = 1 - (short) (*end_char);

    /* Encode the values */

    c = 0;

    long_to_char(record, &defaults->magic, big_endian, &c);

    long_to_char(record, &defaults->bitmap_matrix.width, big_endian, &c);
    long_to_char(record, &defaults->bitmap_matrix.height, big_endian, &c);
    long_to_char(record, &defaults->bitmap_matrix.bits_per_pixel, big_endian, &c);
    long_to_char(record, &defaults->bitmap_matrix.bytes_per_row, big_endian, &c);

    enum_value = defaults->bitmap_matrix.num_type;
    short_to_char(record, &enum_value, big_endian, &c);
    enum_value = defaults->bitmap_matrix.data_type;
    short_to_char(record, &enum_value, big_endian, &c);
    enum_value = defaults->color_pref;
    short_to_char(record, &enum_value, big_endian, &c);
    return STAT_SUCCESS;

}

/************************************************************************

  A function to encode the MUSE BASEMAP structure

*************************************************************************/

#if XVT_CC_PROTO
ERRSTATUS
encode_basemap(unsigned char *record, BASEMAP * basemap)
#else

ERRSTATUS
encode_basemap(record, basemap)
    unsigned char  *record;	/* the header record                    */
    BASEMAP        *basemap;	/* the header structure                 */
#endif

{
    long            c;
    short           end_check = 1;
    char           *end_char;
    short           big_endian;
    short           enum_value;
    short           bool_value;
    long            ptr_value;

    /* Check for endian type */

    end_char = (char *) &end_check;
    big_endian = 1 - (short) (*end_char);

    /* Encode the values */

    c = 0;

    long_to_char(record, &basemap->magic, big_endian, &c);

    enum_value = basemap->product;
    short_to_char(record, &enum_value, big_endian, &c);

    ptr_value = (basemap->area) ? (long) SZ_RASTER : (long) NULL;
    long_to_char(record, &ptr_value, big_endian, &c);
    ptr_value = (basemap->bit_map) ? (long) SZ_RASTER : (long) NULL;
    long_to_char(record, &ptr_value, big_endian, &c);

    double_to_char(record, &basemap->render_info.shade_threshold, big_endian, &c);
    double_to_char(record, &basemap->render_info.precision, big_endian, &c);
    long_to_char(record, &basemap->render_info.sunx, big_endian, &c);
    long_to_char(record, &basemap->render_info.suny, big_endian, &c);

    double_to_char(record, &basemap->zoom, big_endian, &c);

    bool_value = basemap->needs_refresh;
    short_to_char(record, &bool_value, big_endian, &c);
    bool_value = basemap->needs_redraw;
    short_to_char(record, &bool_value, big_endian, &c);
    bool_value = basemap->needs_reload;
    short_to_char(record, &bool_value, big_endian, &c);
    bool_value = basemap->warp;
    short_to_char(record, &bool_value, big_endian, &c);
    return STAT_SUCCESS;

}

/************************************************************************

  A function to encode the MUSE GRAT structure

*************************************************************************/

#if XVT_CC_PROTO
ERRSTATUS
encode_grat(unsigned char *record, GRAT_DATA * grat)
#else
ERRSTATUS
encode_grat(record, grat)
    unsigned char  *record;	/* the header record  */
    GRAT_DATA      *grat;
#endif

{
    long            c;
    short           end_check = 1;
    char           *end_char;
    short           big_endian;
    short           enum_value;
    short           bool_value;

    /* Check for endian type */

    end_char = (char *) &end_check;
    big_endian = 1 - (short) (*end_char);

    /* Encode the values */

    c = 0;

    long_to_char(record, &grat->color, big_endian, &c);
    /* SR 10/09 */
    long_to_char(record, &grat->magic, big_endian, &c);
    long_to_char(record, &grat->bit_flags, big_endian, &c);

    /* border major */
    bool_value = grat->contains_valid_data;
    short_to_char(record, &bool_value, big_endian, &c);

    bool_value = grat->border_enable;
    short_to_char(record, &bool_value, big_endian, &c);

    bool_value = grat->border_major_enable;
    short_to_char(record, &bool_value, big_endian, &c);

    double_to_char(record, &grat->border_major_lat, big_endian, &c);

    enum_value = grat->border_major_lat_unit;
    short_to_char(record, &enum_value, big_endian, &c);

    double_to_char(record, &grat->border_major_lon, big_endian, &c);

    enum_value = grat->border_major_lon_unit = enum_value;
    short_to_char(record, &enum_value, big_endian, &c);

    long_to_char(record, &grat->border_major_color, big_endian, &c);



    /* border minor */
    bool_value = grat->border_minor_enable;
    short_to_char(record, &bool_value, big_endian, &c);

    double_to_char(record, &grat->border_minor_lat, big_endian, &c);

    enum_value = grat->border_minor_lat_unit;
    short_to_char(record, &enum_value, big_endian, &c);

    double_to_char(record, &grat->border_minor_lon, big_endian, &c);

    enum_value = grat->border_minor_lon_unit;
    short_to_char(record, &enum_value, big_endian, &c);

    long_to_char(record, &grat->border_minor_color, big_endian, &c);



    /* border label */
    bool_value = grat->border_labels_enable;
    short_to_char(record, &bool_value, big_endian, &c);

    double_to_char(record, &grat->border_labels_lat, big_endian, &c);

    enum_value = grat->border_labels_lat_unit;
    short_to_char(record, &enum_value, big_endian, &c);

    double_to_char(record, &grat->border_labels_lon, big_endian, &c);

    enum_value = grat->border_labels_lon_unit;
    short_to_char(record, &enum_value, big_endian, &c);

    long_to_char(record, &grat->border_labels_color, big_endian, &c);



    /* grat major */
    bool_value = grat->grat_enable;
    short_to_char(record, &bool_value, big_endian, &c);

    bool_value = grat->grat_major_enable;
    short_to_char(record, &bool_value, big_endian, &c);

    enum_value = grat->grat_major;
    short_to_char(record, &enum_value, big_endian, &c);

    record[c++] = grat->grat_major_ascii_code;

    double_to_char(record, &grat->grat_major_lat, big_endian, &c);

    enum_value = grat->grat_major_lat_unit;
    short_to_char(record, &enum_value, big_endian, &c);

    double_to_char(record, &grat->grat_major_lon, big_endian, &c);

    enum_value = grat->grat_major_lon_unit;
    short_to_char(record, &enum_value, big_endian, &c);

    long_to_char(record, &grat->grat_major_color, big_endian, &c);


    /* grat_minor */
    bool_value = grat->grat_minor_enable;
    short_to_char(record, &bool_value, big_endian, &c);

    enum_value = grat->grat_minor;
    short_to_char(record, &enum_value, big_endian, &c);

    record[c++] = grat->grat_minor_ascii_code;

    double_to_char(record, &grat->grat_minor_lat, big_endian, &c);

    enum_value = grat->grat_minor_lat_unit;
    short_to_char(record, &enum_value, big_endian, &c);

    double_to_char(record, &grat->grat_minor_lon, big_endian, &c);

    enum_value = grat->grat_minor_lon_unit;
    short_to_char(record, &enum_value, big_endian, &c);

    long_to_char(record, &grat->grat_minor_color, big_endian, &c);

    return STAT_SUCCESS;
}

#if 0
/************************************************************************

  A function to encode the MUSE VEC_DATA structure

*************************************************************************/

#if XVT_CC_PROTO
ERRSTATUS
encode_vec_data(unsigned char *record, VEC_DATA * vec_data)
#else

ERRSTATUS
encode_vec_data(record, vec_data)
    unsigned char  *record;	/* the header record                    */
    VEC_DATA       *vec_data;	/* the header structure                 */
#endif

{
    long            c;
    short           end_check = 1;
    char           *end_char;
    short           big_endian;
    short           bool_value;
    long            ptr_value;

    /* Check for endian type */
    end_char = (char *) &end_check;
    big_endian = 1 - (short) (*end_char);

    /* Encode the values */
    c = 0;
    /* SR 10/09 */
    long_to_char(record, &vec_data->magic, big_endian, &c);
    long_to_char(record, &vec_data->bit_flags, big_endian, &c);

    bool_value = vec_data->contains_valid_data;
    short_to_char(record, &bool_value, big_endian, &c);

    double_to_char(record, &vec_data->edge_width, big_endian, &c);

    long_to_char(record, &vec_data->edge_color, big_endian, &c);

    long_to_char(record, &vec_data->text_color, big_endian, &c);

    long_to_char(record, &vec_data->point_color, big_endian, &c);

    /* ptr_value = (long)vec_data->vector; */
    ptr_value = vec_data->vector ? (long) SZ_VECTOR : (long) NULL;
    long_to_char(record, &ptr_value, big_endian, &c);

    return STAT_SUCCESS;
}
#endif
