/*
 * Copyright 1994, 1995 Vectaport Inc., Cartoactive Systems
 *
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee, provided
 * that the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation, and that the names of the copyright holders not be used in
 * advertising or publicity pertaining to distribution of the software
 * without specific, written prior permission.  The copyright holders make
 * no representations about the suitability of this software for any purpose.
 * It is provided "as is" without express or implied warranty.
 *
 * THE COPYRIGHT HOLDERS DISCLAIM ALL WARRANTIES WITH REGARD TO THIS
 * SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS.
 * IN NO EVENT SHALL THE COPYRIGHT HOLDERS BE LIABLE FOR ANY SPECIAL,
 * INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING
 * FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT,
 * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
 * WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 * 
 */

#include <Attribute/attribute.h>
#include <Attribute/attrlist.h>
#include <Attribute/attrvalue.h>

#include <Vpf/vpfcov.h>
#include <Vpf/vpfdb.h>
#include <Vpf/vpflib.h>
#include <Vpf/vpffeature.h>
#include <Vpf/vpffclass.h>
#include <Map/mapprim.h>

#include <limits.h>

#include <float.h>

#include <iostream.h>

#include <string.h>

/*****************************************************************************/

char VpfFeature::_desc[256];

VpfFeature::VpfFeature(VpfFeatureClass* fc, int id, MapFeatureType t, int ft,
		       row_type r)
: MapFeature(fc,id,t) {
    _fclass = fc;
    _ftype = ft;
    _row = r;
    create_attrlist();
    _extent = nil;
}

VpfFeature::~VpfFeature() {
    /* Inspired by DosVpfview\'s free_row */
    if (_row) {
	for (int i = 0; i < feature_class()->ncolumns(); i++) {
	    if (_row[i].ptr)
		free(_row[i].ptr);
	}
	free(_row); 
    }
    delete _extent;
}

boolean VpfFeature::definition(ostream& out) {
#if 0
    int i;
    switch(type()) {
    case MapFeature::PointType:
	{
	    MapPoint* point = point_primitive();
	    if (!point) break;
	    out << "(" << point->xorigin() << "," << point->yorigin()
		<< ")";
	}
	break;
    case MapFeature::LineType:
	{
	    MapEdge* edge = edge_primitive();
	    if (!edge) break;
	    int npts = edge->npts();
	    const float* fx = edge->xpoints();
	    const float* fy = edge->ypoints();
	    for (i=0; i<npts; i++) {
		out << "(" << fx[i] << "," << fy[i] << ")";
		if (i+1 < npts ) out << ",";
	    }
	}
	break;
    case MapFeature::AreaType:
	{
	    MapFace* face = face_primitive();
	    if (!face) break;
	    int npts = face->npts();
	    const float* fx = face->xpoints();
	    const float* fy = face->ypoints();
	    for (i=0; i<npts; i++) {
		out << "(" << fx[i] << "," << fy[i] << ")";
		if (i+1 < npts ) out << ",";
	    }
	}
	break;
    case MapFeature::TextType:
	{
	    MapText* text = text_primitive();
	    if (!text) break;
	    out << "(" << text->xorigin() << "," << text->yorigin()
		<< ")";
	}
	break;
    case MapFeature::JunctionType:
    case MapFeature::RasterType:
    case MapFeature::DemType:
    default:
	cerr << "Unsupported (as of yet) map feature\n";
	break;
    }
#endif
    return attr_list(out);
}

void VpfFeature::create_attrlist() {
    _attrlist = new AttributeList();
    char dbuf[256], rbuf[256], wbuf[256];
    void* value = &rbuf;
    void* rvalue;
    long int valuecount;
    rbuf[0] = 0;
    dbuf[0] = 0;
    for (long int i = 0; i < feature_class()->ncolumns(); i++) {
	rvalue = row_element(i,value,&valuecount);
	VpfColumnHeader* col = feature_class()->column(i);
	const char* colname = col->name();
	if (colname[0] == ',' && colname[1] == ':')
	  colname = &colname[2];
	AttributeValue* val = new AttributeValue();
	char type = col->type();
	switch (type) {
	case 'X':
	    break;
	case 'T':
	    if (strcmp(colname,"POPYREG")==0 || strcmp(colname,"POPYCOUN")==0) {
		char* desc = feature_class()->coverage()->char_value_description
		    (colname, (char*)rvalue);
		if (desc) {
		    rightjust(desc);
		    val->type(AttributeValue::StringType);
		    val->string_ref() = symbol_add(desc);
		    free(desc);
		}
	    }
	    else if (valuecount == 1) {
		val->type(AttributeValue::CharType);
		val->char_ref() = rbuf[0];
	    } else {
		rightjust((char *)rvalue);
		val->type(AttributeValue::StringType);
		val->string_ref() = symbol_add((char *)rvalue);
		free(rvalue);
	    }
	    break;
	case 'I':
	    val->type(AttributeValue::LongType);
	    val->long_ref() = *(long*)value;
	    break;
	case 'S':
	    val->type(AttributeValue::ShortType);
	    val->short_ref() = *(short*)value;
	    break;
	case 'F':
	    val->type(AttributeValue::FloatType);
	    val->float_ref() = *(float*)value;
	    break;
	case 'R':
	    val->type(AttributeValue::DoubleType);
	    val->double_ref() = *(double*)value;
	    break;
	}
	_attrlist->add_attr(colname, val);
    }
}

boolean VpfFeature::attr_list(ostream& out) {
    char dbuf[256], rbuf[256], wbuf[256];
    void* value = &rbuf;
    void* rvalue;
    long int valuecount;
    rbuf[0] = 0;
    dbuf[0] = 0;
    for (long int i = 0; i < feature_class()->ncolumns(); i++) {
	rvalue = row_element(i,value,&valuecount);
	VpfColumnHeader* col = feature_class()->column(i);
	const char* colname = col->name();
	char type = col->type();
	switch (type) {
	case 'X':
	    break;
	case 'T':
	    if (strcmp(colname,"POPYREG")==0 || strcmp(colname,"POPYCOUN")==0) {
		char* desc = feature_class()->coverage()->char_value_description
		    (colname, (char*)rvalue);
		if (desc) {
		    rightjust(desc);
		    out << " :" << colname << " \"" << desc << "\"";
		    free(desc);
		}
	    }
	    else if (valuecount == 1) {
	      out << " :" << colname << " \"" << rbuf[0] << "\"";
	    } else {
		rightjust((char *)rvalue);
		out << " :" << colname << " \"" << (char*)rvalue << "\"";
		free(rvalue);
	    }
	    break;
	case 'I':
	    out << " :" << colname << " " << *(long*)value;
	    break;
	case 'S':
	    out << " :" << colname << " " << *(short*)value;
	    break;
	case 'F':
	    out << " :" << colname << " " << *(float*)value;
	    break;
	case 'R':
	    out << " :" << colname << " " << *(double*)value;
	    break;
	case 'C':
#if 0
	    if (valuecount == 1) {
		sprintf(wbuf, "%s: (%f,%f)  ", col->name(),
			((coordinate_type*)rbuf)->x, ((coordinate_type*)rbuf)->y);
		strcat(dbuf, wbuf);
	    } else {
		sprintf(wbuf, "%s: ", col->name());
		strcat(dbuf, wbuf);
		for (int j = 0; j < valuecount; j++) {
		    sprintf(wbuf, "(%f,%f) ", ((coordinate_type*)rvalue)[j].x,
			    ((coordinate_type*)rvalue)[j].y);
		    strcat(dbuf, wbuf);
		}
		strcat(dbuf, " ");
		free(rvalue);
	    }
#endif
	    break;
	case 'Z':
#if 0
	    if (valuecount == 1) {
		sprintf(wbuf, "%s: (%f,%f,%f)  ", col->name(),
			((tri_coordinate_type*)rbuf)->x, ((tri_coordinate_type*)rbuf)->y,
			((tri_coordinate_type*)rbuf)->z);
		strcat(dbuf, wbuf);
	    } else {
		sprintf(wbuf, "%s: ", col->name());
		strcat(dbuf, wbuf);
		for (int j = 0; j < valuecount; j++) {
		    sprintf(wbuf, "(%f,%f,%f) ", ((tri_coordinate_type*)rvalue)[j].x,
			    ((tri_coordinate_type*)rvalue)[j].y,
			    ((tri_coordinate_type*)rvalue)[j].z);
		    strcat(dbuf, wbuf);
		}
		strcat(dbuf, " ");
		free(rvalue);
	    }
#endif
	    break;
	case 'B':
#if 0
	    if (valuecount == 1) {
		sprintf(wbuf, "%s: (%f,%f)  ", col->name(),
			((double_coordinate_type*)rbuf)->x, ((double_coordinate_type*)rbuf)->y);
		strcat(dbuf, wbuf);
	    } else {
		sprintf(wbuf, "%s: ", col->name());
		strcat(dbuf, wbuf);
		for (int j = 0; j < valuecount; j++) {
		    sprintf(wbuf, "(%f,%f) ", ((double_coordinate_type*)rvalue)[j].x,
			    ((double_coordinate_type*)rvalue)[j].y);
		    strcat(dbuf, wbuf);
		}
		strcat(dbuf, " ");
		free(rvalue);
	    }
	    break;
	case 'Y':
	    if (valuecount == 1) {
		sprintf(wbuf, "%s: (%f,%f,%f)  ", col->name(),
			((double_tri_coordinate_type*)rbuf)->x,
			((double_tri_coordinate_type*)rbuf)->y,
			((double_tri_coordinate_type*)rbuf)->z);
		strcat(dbuf, wbuf);
	    } else {
		sprintf(wbuf, "%s: ", col->name());
		strcat(dbuf, wbuf);
		for (int j = 0; j < valuecount; j++) {
		    sprintf(wbuf, "(%f,%f,%f) ", ((double_tri_coordinate_type*)rvalue)[j].x,
			    ((double_tri_coordinate_type*)rvalue)[j].y,
			    ((double_tri_coordinate_type*)rvalue)[j].z);
		    strcat(dbuf, wbuf);
		}
		strcat(dbuf, " ");
		free(rvalue);
	    }
#endif
	    break ;
	case 'D':
#if 0
	    if (valuecount == 1) {
		char date[40];
		format_date(rbuf,date);
		sprintf(wbuf, "%s: %s  ", col->name(), date);
		strcat(dbuf, wbuf);
	    } else {
		char date[40];
		sprintf(wbuf, "%s: ", col->name());
		strcat(dbuf, wbuf);
		for (int j = 0; j < valuecount; j++) {
		    format_date(((date_type*)*rbuf)[j],date);
		    sprintf(wbuf, "%s ", date);
		    strcat(dbuf, wbuf);
		}
		strcat(dbuf, " ");
		free(rvalue);
	    }
#endif
	    break;
	case 'K':  /* ID Triplet */
#if 0
	    if (valuecount == 1) {
		sprintf(wbuf, "%s: (%ld,%ld,%ld)  ", col->name(),
			((id_triplet_type*)rbuf)->id,
			((id_triplet_type*)rbuf)->tile,
			((id_triplet_type*)rbuf)->exid);
		strcat(dbuf, wbuf);
	    } else {
		sprintf(wbuf, "%s: ", col->name());
		strcat(dbuf, wbuf);
		for (int j = 0; j < valuecount; j++) {
		    sprintf(wbuf, "(%ld,%ld,%ld) ",
			    ((id_triplet_type*)rvalue)[j].id,
			    ((id_triplet_type*)rvalue)[j].tile,
			    ((id_triplet_type*)rvalue)[j].exid);
		    strcat(dbuf, wbuf);
		}
		strcat(dbuf, " ");
		free(rvalue);
	    }
#endif
	    break;
	}
    }
    return out.good();
}

/* Inspired by DosVpfview\'s get_table_element */

void* VpfFeature::row_element( long int colnumber, void *value, long int  *count ) const {
    long int   col;
    char     * tptr;
    void     * retvalue;
    
    retvalue = NULL;
    col = colnumber;
    if ((col < 0) || (col >= feature_class()->ncolumns())) {
	fprintf(stderr,"VpfFeature::RowElement: Invalid column number %d\n",
		colnumber);
	return NULL;
    }
    
    if (!_row) return NULL;
    
    VpfColumnHeader* colheader = feature_class()->column(col);
    char type = colheader->type();
    long int valuecount = colheader->count();
    switch (type) {
    case 'X':
	retvalue = NULL;
	break;
    case 'T':
	if (valuecount == 1) {
	    memcpy(value,_row[col].ptr,sizeof(char));
	} else {
	    retvalue = (char *)vpfmalloc((_row[col].count+1)*sizeof(char));
	    tptr = (char *)vpfmalloc((_row[col].count+1)*sizeof(char));
	    memcpy(tptr,_row[col].ptr,_row[col].count*sizeof(char));
	    tptr[_row[col].count] = '\0';
	    strcpy((char *)retvalue,tptr);
	    free(tptr);
	}
	break;
    case 'I':
	if (valuecount == 1) {
	    memcpy(value,_row[col].ptr,sizeof(long int));
	} else {
	    retvalue = (long int *)vpfmalloc(_row[col].count*
					     sizeof(long int));
	    memcpy(retvalue,_row[col].ptr,_row[col].count*sizeof(long int));
	}
	break;
    case 'S':
	if (valuecount == 1) {
	    memcpy(value,_row[col].ptr,sizeof(short int));
	} else {
	    retvalue = (short int *)vpfmalloc(_row[col].count*
					      sizeof(short int));
	    memcpy(retvalue,_row[col].ptr,_row[col].count*sizeof(short int));
	}
	break;
    case 'F':
	if (valuecount == 1) {
	    memcpy(value,_row[col].ptr,sizeof(float));
	} else {
	    retvalue = (float *)vpfmalloc(_row[col].count*sizeof(float));
	    memcpy(retvalue,_row[col].ptr,_row[col].count*sizeof(float));
	}
	break;
    case 'R':
	if (valuecount == 1) {
	    memcpy(value,_row[col].ptr,sizeof(double));
	} else {
	    retvalue = (double *)vpfmalloc(_row[col].count*sizeof(double));
	    memcpy(retvalue,_row[col].ptr,_row[col].count*sizeof(double));
	}
	break;
    case 'C':
	if (valuecount == 1) {
	    memcpy(value,_row[col].ptr,sizeof(coordinate_type));
	} else {
	    if (_row[col].ptr) {
		retvalue = (coordinate_type *)malloc(_row[col].count*
						     sizeof(coordinate_type));
		if (retvalue)
		    memcpy(retvalue,_row[col].ptr,_row[col].count*
			   sizeof(coordinate_type));
	    } else {
		retvalue = NULL;
	    }
	}
	break;
    case 'Z':
	if (valuecount == 1) {
	    memcpy(value,_row[col].ptr,sizeof(tri_coordinate_type));
	} else {
	    retvalue = (tri_coordinate_type *)vpfmalloc(_row[col].count*
							sizeof(tri_coordinate_type));
	    memcpy(retvalue,_row[col].ptr,_row[col].count*
		   sizeof(tri_coordinate_type));
	}
	break;
    case 'B':
	if (valuecount == 1) {
	    memcpy(value,_row[col].ptr,sizeof(double_coordinate_type));
	} else {
	    retvalue = (double_coordinate_type *)vpfmalloc(_row[col].count*
							   sizeof(double_coordinate_type));
	    memcpy(retvalue,_row[col].ptr,_row[col].count*
		   sizeof(double_coordinate_type));
	}
	break;
    case 'Y':
	if (valuecount == 1) {
	    memcpy(value,_row[col].ptr,sizeof(double_tri_coordinate_type));
	} else {
	    retvalue = (double_tri_coordinate_type *)vpfmalloc(_row[col].count*
							       sizeof(double_tri_coordinate_type));
	    memcpy(retvalue,_row[col].ptr,_row[col].count*
		   sizeof(double_tri_coordinate_type));
	}
	break ;
    case 'D':
	if (valuecount == 1) {
	    memcpy(value,_row[col].ptr,sizeof(date_type));
	} else {
	    retvalue = (date_type *)vpfmalloc(_row[col].count*
					      sizeof(date_type));
	    memcpy(retvalue,_row[col].ptr,_row[col].count*
		   sizeof(date_type));
	}
	break;
    case 'K':  /* ID Triplet */
	if (valuecount == 1) {
	    memcpy(value,_row[col].ptr,sizeof(id_triplet_type));
	} else {
	    retvalue = (id_triplet_type *)vpfmalloc(_row[col].count*
						    sizeof(id_triplet_type));
	    memcpy(retvalue,_row[col].ptr,_row[col].count*
		   sizeof(id_triplet_type));
	}
	break;
    }
    *count = _row[col].count;
    
    return retvalue;
}

void* VpfFeature::named_row_element( char* colname, void *value, long int  *count ) const {
    long int     col;
    void       * retvalue;
    
    col = feature_class()->column_pos(colname);
    if (col < 0) {
	cerr << "VpfFeature::NamedRowElement: Invalid field name <" << colname << "\n";
	return NULL;
    }
    retvalue = row_element( col, value, count );
    return retvalue;
}

extern "C" void format_date( date_type date, char *fmtdate );

const char* VpfFeature::describe() const {
    char dbuf[256], rbuf[256], wbuf[256];
    void* value = &rbuf;
    void* rvalue;
    long int valuecount;
    rbuf[0] = 0;
    dbuf[0] = 0;
    for (long int i = 0; i < feature_class()->ncolumns(); i++) {
	rvalue = row_element(i,value,&valuecount);
	VpfColumnHeader* col = feature_class()->column(i);
	const char* colname = col->name();
	char type = col->type();
	switch (type) {
	case 'X':
	    break;
	case 'T':
	    if (strcmp(colname,"POPYREG")==0 || strcmp(colname,"POPYCOUN")==0) {
		char* desc = feature_class()->coverage()->char_value_description
		    (colname, (char*)rvalue);
		if (desc) {
		    rightjust(desc);
		    sprintf(wbuf, "%s: %s  ", col->name(), desc);
		    strcat(dbuf, wbuf);
		    free(desc);
		}
		else {
		    sprintf(wbuf, "%s: (null)  ", col->name());
		    strcat(dbuf, wbuf);
		}
	    }
	    else if (valuecount == 1) {
		sprintf(wbuf, "%s: %c  ", col->name(), rbuf[0]);
		strcat(dbuf, wbuf);
	    } else {
		rightjust((char *)rvalue);
		sprintf(wbuf, "%s: %s  ", col->name(), (char *)rvalue);
		strcat(dbuf, wbuf);
		free(rvalue);
	    }
	    break;
	case 'I':
	    if (strstr(colname,"TYPE")) {
		// look it up in int.vdt
		char* desc = feature_class()->coverage()->int_value_description(colname, *(long int*)value);
		if (desc) {
		    rightjust(desc);
		    sprintf(wbuf, "%s: %s  ", col->name(), desc);
		    strcat(dbuf, wbuf);
		    free(desc);
		}
		else {
		    sprintf(wbuf, "%s: (null)  ", col->name());
		    strcat(dbuf, wbuf);
		}
	    }
	    else
		if (valuecount == 1) {
		    if (*(long int*)value == MISSING_VALUE)
			sprintf(wbuf, "%s: (null)  ", col->name());
		    else
			sprintf(wbuf, "%s: %d  ", col->name(), *(long int*)value);
		    strcat(dbuf, wbuf);
		} else {
		    sprintf(wbuf, "%s: ", col->name());
		    strcat(dbuf, wbuf);
		    for (int j = 0; j < valuecount; j++) {
			sprintf(wbuf, "%d ", ((long int*)rvalue)[j]);
			strcat(dbuf, wbuf);
		    }
		    strcat(dbuf, " ");
		    free(rvalue);
		}
	    break;
	case 'S':
	    if (valuecount == 1) {
		sprintf(wbuf, "%s: %d  ", col->name(), *(short int*)value);
		strcat(dbuf, wbuf);
	    } else {
		sprintf(wbuf, "%s: ", col->name());
		strcat(dbuf, wbuf);
		for (int j = 0; j < valuecount; j++) {
		    sprintf(wbuf, "%d ", ((short int*)rvalue)[j]);
		    strcat(dbuf, wbuf);
		}
		strcat(dbuf, " ");
		free(rvalue);
	    }
	    break;
	case 'F':
	    if (valuecount == 1) {
		sprintf(wbuf, "%s: %f  ", col->name(), *(float*)value);
		strcat(dbuf, wbuf);
	    } else {
		sprintf(wbuf, "%s: ", col->name());
		strcat(dbuf, wbuf);
		for (int j = 0; j < valuecount; j++) {
		    sprintf(wbuf, "%f ", ((float*)rvalue)[j]);
		    strcat(dbuf, wbuf);
		}
		strcat(dbuf, " ");
		free(rvalue);
	    }
	    break;
	case 'R':
	    if (valuecount == 1) {
		sprintf(wbuf, "%s: %f  ", col->name(), *(double*)value);
		strcat(dbuf, wbuf);
	    } else {
		sprintf(wbuf, "%s: ", col->name());
		strcat(dbuf, wbuf);
		for (int j = 0; j < valuecount; j++) {
		    sprintf(wbuf, "%f ", ((double*)rvalue)[j]);
		    strcat(dbuf, wbuf);
		}
		strcat(dbuf, " ");
		free(rvalue);
	    }
	    break;
	case 'C':
	    if (valuecount == 1) {
		sprintf(wbuf, "%s: (%f,%f)  ", col->name(),
			((coordinate_type*)rbuf)->x, ((coordinate_type*)rbuf)->y);
		strcat(dbuf, wbuf);
	    } else {
		sprintf(wbuf, "%s: ", col->name());
		strcat(dbuf, wbuf);
		for (int j = 0; j < valuecount; j++) {
		    sprintf(wbuf, "(%f,%f) ", ((coordinate_type*)rvalue)[j].x,
			    ((coordinate_type*)rvalue)[j].y);
		    strcat(dbuf, wbuf);
		}
		strcat(dbuf, " ");
		free(rvalue);
	    }
	    break;
	case 'Z':
	    if (valuecount == 1) {
		sprintf(wbuf, "%s: (%f,%f,%f)  ", col->name(),
			((tri_coordinate_type*)rbuf)->x, ((tri_coordinate_type*)rbuf)->y,
			((tri_coordinate_type*)rbuf)->z);
		strcat(dbuf, wbuf);
	    } else {
		sprintf(wbuf, "%s: ", col->name());
		strcat(dbuf, wbuf);
		for (int j = 0; j < valuecount; j++) {
		    sprintf(wbuf, "(%f,%f,%f) ", ((tri_coordinate_type*)rvalue)[j].x,
			    ((tri_coordinate_type*)rvalue)[j].y,
			    ((tri_coordinate_type*)rvalue)[j].z);
		    strcat(dbuf, wbuf);
		}
		strcat(dbuf, " ");
		free(rvalue);
	    }
	    break;
	case 'B':
	    if (valuecount == 1) {
		sprintf(wbuf, "%s: (%f,%f)  ", col->name(),
			((double_coordinate_type*)rbuf)->x, ((double_coordinate_type*)rbuf)->y);
		strcat(dbuf, wbuf);
	    } else {
		sprintf(wbuf, "%s: ", col->name());
		strcat(dbuf, wbuf);
		for (int j = 0; j < valuecount; j++) {
		    sprintf(wbuf, "(%f,%f) ", ((double_coordinate_type*)rvalue)[j].x,
			    ((double_coordinate_type*)rvalue)[j].y);
		    strcat(dbuf, wbuf);
		}
		strcat(dbuf, " ");
		free(rvalue);
	    }
	    break;
	case 'Y':
	    if (valuecount == 1) {
		sprintf(wbuf, "%s: (%f,%f,%f)  ", col->name(),
			((double_tri_coordinate_type*)rbuf)->x,
			((double_tri_coordinate_type*)rbuf)->y,
			((double_tri_coordinate_type*)rbuf)->z);
		strcat(dbuf, wbuf);
	    } else {
		sprintf(wbuf, "%s: ", col->name());
		strcat(dbuf, wbuf);
		for (int j = 0; j < valuecount; j++) {
		    sprintf(wbuf, "(%f,%f,%f) ", ((double_tri_coordinate_type*)rvalue)[j].x,
			    ((double_tri_coordinate_type*)rvalue)[j].y,
			    ((double_tri_coordinate_type*)rvalue)[j].z);
		    strcat(dbuf, wbuf);
		}
		strcat(dbuf, " ");
		free(rvalue);
	    }
	    break ;
	case 'D':
	    if (valuecount == 1) {
		char date[40];
		format_date(rbuf,date);
		sprintf(wbuf, "%s: %s  ", col->name(), date);
		strcat(dbuf, wbuf);
	    } else {
		char date[40];
		sprintf(wbuf, "%s: ", col->name());
		strcat(dbuf, wbuf);
		for (int j = 0; j < valuecount; j++) {
		    format_date(((date_type*)*rbuf)[j],date);
		    sprintf(wbuf, "%s ", date);
		    strcat(dbuf, wbuf);
		}
		strcat(dbuf, " ");
		free(rvalue);
	    }
	    break;
	case 'K':  /* ID Triplet */
	    if (valuecount == 1) {
		sprintf(wbuf, "%s: (%ld,%ld,%ld)  ", col->name(),
			((id_triplet_type*)rbuf)->id,
			((id_triplet_type*)rbuf)->tile,
			((id_triplet_type*)rbuf)->exid);
		strcat(dbuf, wbuf);
	    } else {
		sprintf(wbuf, "%s: ", col->name());
		strcat(dbuf, wbuf);
		for (int j = 0; j < valuecount; j++) {
		    sprintf(wbuf, "(%ld,%ld,%ld) ",
			    ((id_triplet_type*)rvalue)[j].id,
			    ((id_triplet_type*)rvalue)[j].tile,
			    ((id_triplet_type*)rvalue)[j].exid);
		    strcat(dbuf, wbuf);
		}
		strcat(dbuf, " ");
		free(rvalue);
	    }
	    break;
	}
    }
    rightjust(dbuf);
    strcpy(_desc, dbuf);
    return _desc;
}

VpfExtent* VpfFeature::extent() {
  if (!_extent) {
    switch (_type) {
    case LineType:
    case AreaType:
	char path[256];
	long int n;
	float xmin,ymin,xmax,ymax;
	int XMIN_, YMIN_, XMAX_, YMAX_;
	vpf_table_type table;
	row_type row;
	extent_type fextent;
	strcpy( path, feature_class()->coverage()->path() );
	if (_type == LineType) 
	    strcat( path, "/ebr." );
	else
	    strcat( path, "/fbr." );
	table = vpf_open_table( path, disk, "rb", NULL );
	row = read_row( id(), table );
	XMIN_ = table_pos( "XMIN", table );
	YMIN_ = table_pos( "YMIN", table );
	XMAX_ = table_pos( "XMAX", table );
	YMAX_ = table_pos( "YMAX", table );
	get_table_element( XMIN_, row, table, &xmin, &n );
	get_table_element( YMIN_, row, table, &ymin, &n );
	get_table_element( XMAX_, row, table, &xmax, &n );
	get_table_element( YMAX_, row, table, &ymax, &n );
	fextent.x1 = xmin;
	fextent.y1 = ymin;
	fextent.x2 = xmax;
	fextent.y2 = ymax;
	free_row( row, table );
	vpf_close_table( &table );
	_extent = new VpfExtent(fextent);
	break;
    default:
	break;
    }
  }
  return _extent;
}

float VpfFeature::unprojscale() { return 100000.0; }

char* VpfFeature::text_attr(char* attrname) {
    long int col;
    long int count;
    col = feature_class()->column_pos(attrname);
    if (col >= 0 && col < feature_class()->ncolumns() &&
	feature_class()->column(col)->type() == 'T')
	{
	    char value;
	    char* retval = (char*)named_row_element( attrname, &value, &count);
	    if (count == 1) {
		retval = new char[2];
		retval[0] = (char)value;
		retval[1] = '\0';
	    }
	    return retval;
	}
    return "";
}

float VpfFeature::float_attr(char* attrname) {
    long int col;
    long int count;
    col = feature_class()->column_pos(attrname);
    if (col >= 0 && col < feature_class()->ncolumns() &&
	feature_class()->column(col)->type() == 'F')
	{
	    float value;
	    named_row_element(attrname, &value, &count);
	    return value;
	}
    return FLT_MIN;
}

double VpfFeature::double_attr(char* attrname) {
    long int col;
    long int count;
    col = feature_class()->column_pos(attrname);
    if (col >= 0 && col < feature_class()->ncolumns() &&
	feature_class()->column(col)->type() == 'R')
	{
	    double value;
	    named_row_element(attrname, &value, &count);
	    return value;
	}
    return DBL_MIN;
}

short int VpfFeature::short_attr(char* attrname) {
    long int col;
    long int count;
    col = feature_class()->column_pos(attrname);
    if (col >= 0 && col < feature_class()->ncolumns() &&
	feature_class()->column(col)->type() == 'S')
	{
	    short int value;
	    named_row_element(attrname, &value, &count);
	    return value;
	}
    return SHRT_MIN;
}

long int VpfFeature::long_attr(char* attrname) {
    long int col;
    long int count;
    col = feature_class()->column_pos(attrname);
    if (col >= 0 && col < feature_class()->ncolumns() &&
	feature_class()->column(col)->type() == 'I')
	{
	    long int value;
	    named_row_element(attrname, &value, &count);
	    return value;
	}
    return LONG_MIN;
}
